import os.path

BASEDIR = os.path.expanduser("~/devel/")

import sys

sys.path.append(BASEDIR)

import PySPG

# Lee los labels de las columnas
from plotdyconf import *

for kcol, kerr, colname in col_contents:
    p2 = PySPG.MultiAgrizer(open("param.dat").readlines(), lastvar)
    p2.xlabel = xlabel
    p2.ylabel = colname

    p2.plottype = "xydy"
    p2.ycol = kcol
    p2.zcol = kerr

    p2.doit(f"{str(kcol)}err.agr", "out.dat")



#
#  This script plots all the specified files
#  outputed by a {}-something directive in a recoursive-directory
#  But it does not plot the file PLOTS AN HISTOGRAM CONSTRUCTED
#  FROM THE SERIE
#

# Specifies the grace format of the x axis
#

xlabel = "\\f{Times-Italic}t\\sk"
#
# Specifies the name of the last variable considered
# before plotting
#

histvec = ["{size}.dat"]


# configuration File
inputFile = "param.dat"

#
# If the files containing the data have a time serie that must be differenced first
# set this to something different to zero
# if you want to construct the histogram only with the raw data of the
# file leave it as is
mustComputeDiference = 0


# WARNING YOU MUST SPECIFY THE DIRECTORY WHERE PySPG LIVES
PySPGPATH = "/t/users1/tessonec/devel/"


import os.path
import sys

try:
    os.path.isdir(PySPGPATH)
    sys.path.append(os.path.expanduser(PySPGPATH))
except:
    sys.stderr.write(f"error! '{PySPGPATH}' directory NOT FOUND\n")
    sys.exit(2)

try:
    from PySPG import *
except:
    sys.stderr.write("couldn't import PySPG package, check PySPGPATH variable\n")
    sys.stderr.write("and verify that PySPG lives there\n")
    sys.stderr.write(f"actual value: '{PySPGPATH}'\n")

    sys.exit(2)


for histvar in histvec:
    p2 = Agrizer(open(inputFile).readlines())

    p2.xlabel = xlabel
    p2.ylabel = "\\f{Times-Italic}P\\f{}(%s\\f{}\\N)" % xlabel

    p2.dohist(histvar)



#
#:::~ Author: Claudio J. Tessone <tessonec@imedea.uib.es>
#:::~ Release: Wed Oct  1 20:27:38 CEST 2003
#:::~ Distributed under GNU/GPL License
#
#


columnX = 12

import os.path
import sys

#
# WARNING YOU MUST SPECIFY THE DIRECTORY WHERE PySPG LIVES
PySPGPATH = "~/devel/"
#
# If you do not plan to change the name of ther executable,
# you can specify it here (obvioulsy it is a string)
paramFile = "param.dat"


def usage():
    print(f"usage is: {sys.argv[0]} [options] executable")
    print()
    print(
        ' -f param-file.dat : Reads parameters from param-file.dat \n\t\t (by default, "param.dat") '
    )
    print(
        ' -i in-file.dat : The name of the file read by the executable \n\t\t(by default, "in.dat") '
    )
    print(" - : Reads parameters from standard input ")
    print("")
    print('Repetedly executes "executable". ', end=" ")
    print(
        'If you do not plan to change the name \nof the executable, you can directly modify the "execFile" variable \ninside this script'
    )
    print()
    print()


try:
    os.path.isdir(PySPGPATH)
    sys.path.append(os.path.expanduser(PySPGPATH))
except:
    sys.stderr.write(f"Error! '{PySPGPATH}' directory NOT FOUND\n")
    sys.exit(2)

try:
    from PySPG import *
except:
    sys.stderr.write("Couldn't import PySPG package, check PySPGPATH variable\n")
    sys.stderr.write("And verify that PySPG lives there\n")
    sys.stderr.write(f"actual value: '{PySPGPATH}'\n")

    sys.exit(2)

try:
    lsLines = open(paramFile).readlines()
except:
    sys.stderr.write(f"Error! Couldn't open '{paramFile}' file\n")
    sys.exit(2)


for i in range(1, 9):
    print(i)
    mc = MeanCalculation(lsLines)
    mc.doit("out.dat", 0, i)



#
#  This script plots from a recursive-directory all the requested columns
#  plotting ONE COLUMN IN ONE FILE
#


#
# Specifies the grace format of the x axis
#

xlabel = "\\f{Symbol}g"
#
# Specifies the name of the last variable considered
# before plotting
#
lastvar = "size"
size = 3
#
# Specifies the columns to be plotted
# and its label (in grace format)
#

#
# The output of ct-sr-linfhn is special:
# the first __size__ columns give SNR
#
col_contents_snr = [([i + 1 for i in range(size)], "\\f{Times-Italic}SNR")]
# the second __size__ columns give SAF
col_contents_saf = [([i + 1 + size for i in range(size)], "\\f{Times-Italic}SAF")]
# the third __size__ columns give the jitter
col_contents_ns = [([i + 1 + 2 * size for i in range(size)], "\\f{Times-Italic}J")]

# the fourth __size__ columns give the number of spikes
col_contents_ns = [([i + 1 + 3 * size for i in range(size)], "\\f{Times-Italic}N\\sS")]

#  let's put all the contents together
col_contents = col_contents_snr + col_contents_saf + col_contents_ns

# configuration File
inputFile = "param.dat"


# WARNING YOU MUST SPECIFY THE DIRECTORY WHERE PySPG LIVES
PySPGPATH = "~/devel/"

import os.path
import sys

try:
    os.path.isdir(PySPGPATH)
    sys.path.append(os.path.expanduser(PySPGPATH))
except:
    sys.stderr.write(f"error! '{PySPGPATH}' directory NOT FOUND\n")
    sys.exit(2)

try:
    from PySPG import *
except:
    sys.stderr.write("couldn't import PySPG package, check PySPGPATH variable\n")
    sys.stderr.write("and verify that PySPG lives there\n")
    sys.stderr.write(f"actual value: '{PySPGPATH}'\n")

    sys.exit(2)


for kcol, colname in col_contents:
    p2 = MultiAgrizer(open(inputFile).readlines(), lastvar)
    p2.xlabel = xlabel
    p2.ylabel = colname

    p2.plottype = "xy"
    p2.ycol = kcol
    p2.xscale = "Logarithmic"
    outname = f"{'-'.join([str(i) for i in kcol])}.agr"
    p2.doit(outname, "out.dat")



"""
:::~ Author: Claudio Juan Tessone <tessonec@imedea.uib.es> (c) 2002-2003

Distributed According to GNU Generic Purpose License (GPL)
Please visit www.gnu.org
"""

from .Agrizer import *
from .Executor import *
from .MatrixPlotter import *
from .MeanCalculation import *
from .MultiAgrizer import *
from .ParamParser import *
from .TeXParser import *



#
#  This script plots all the specified files
#  outputed by a {}-something directive in a recoursive-directory
#

#
# Specifies the grace format of the x axis
#

xlabel = "\\f{Times-Italic}t"
#
# Specifies the name of the last variable considered
# before plotting
#
extension = "psd"
size = 3

histvec = ["%d{gama}.%s" % (i, extension) for i in range(size)]

# configuration File
inputFile = "param.dat"


# WARNING YOU MUST SPECIFY THE DIRECTORY WHERE PySPG LIVES
PySPGPATH = "/t/users1/tessonec/devel/"


for histvar in histvec:

    p2 = Agrizer(open(inputFile).readlines())

    p2.xlabel = xlabel
    p2.ylabel = "\\f{Times-Italic}P\\f{}(%s\\f{})" % xlabel
    p2.yscale = "Logarithmic"
    p2.doit(pattern=histvar)



#
#  This script plots all the specified files
#  outputed by a {}-something directive in a recoursive-directory
#

#
# Specifies the grace format of the x axis
#

xlabel = "\\f{Times-Italic}t"
#
# Specifies the name of the last variable considered
# before plotting
#
extension = "psd"
size = 3

histvec = ["%d{gama}.%s" % (i, extension) for i in range(size)]

# configuration File
inputFile = "param.dat"


# WARNING YOU MUST SPECIFY THE DIRECTORY WHERE PySPG LIVES
PySPGPATH = "/t/users1/tessonec/devel/"


import os.path
import sys

try:
    os.path.isdir(PySPGPATH)
    sys.path.append(os.path.expanduser(PySPGPATH))
except:
    sys.stderr.write(f"error! '{PySPGPATH}' directory NOT FOUND\n")
    sys.exit(2)

try:
    from PySPG import *
except:
    sys.stderr.write("couldn't import PySPG package, check PySPGPATH variable\n")
    sys.stderr.write("and verify that PySPG lives there\n")
    sys.stderr.write(f"actual value: '{PySPGPATH}'\n")

    sys.exit(2)


p2 = Agrizer(open(inputFile).readlines())

p2.xlabel = xlabel
p2.ylabel = "\\f{Times-Italic}P\\f{}(%s\\f{})" % xlabel
p2.yscale = "Logarithmic"
p2.doit(pattern=histvec)



#
# Specifies the name of the last variable considered
# before plotting
#
lastvar = "k"

#
# Specifies the columns to be plotted
# and its label (in grace format)
#
col_contents = [(1, "\\f{Symbol} <r>"), (2, "\\f{Symbol} <dr>"), (3, "U\\sl")]


#
# Specifies the grace format of the x axis
#
xlabel = "\\f{Times-Italic}P"



import os.path
import sys

#
#:::~ Author: Claudio J. Tessone <tessonec@imedea.uib.es>
#:::~ Release: Jan 10 2005
#:::~ Distributed under GNU/GPL License
#


#
# WARNING YOU MUST SPECIFY THE DIRECTORY WHERE PySPG LIVES
PySPGPATH = "~/devel/"
#
# name of the executable that has the main simulation loop
execFile = "ct-ordprm-kuramoto"
# name of the file expected by the executable for its input
inConfFile = "in.kmoto"
# PySPG configuration file
paramFile = "param.dat"

readCIN = False

# Checks whether the PySPG path exists, and adds it to the PATH
try:
    os.path.isdir(PySPGPATH)
    sys.path.append(os.path.expanduser(PySPGPATH))
except:
    sys.stderr.write(f"Error! '{PySPGPATH}' directory NOT FOUND\n")
    sys.exit(2)


# imports the PySPG library
try:
    from PySPG import *
except:
    sys.stderr.write("Couldn't import PySPG package, check PySPGPATH variable\n")
    sys.stderr.write("And verify that PySPG lives there\n")
    sys.stderr.write(f"actual value: '{PySPGPATH}'\n")

    sys.exit(2)

# reads the PySPG parameter file
try:
    lsLines = open(paramFile).readlines()
except:
    sys.stderr.write(f"Error! Couldn't open '{paramFile}' file\n")
    sys.exit(2)


# All the PySPG is done here!
xcutor = Executor(lsLines)
xcutor.doit(execFile, inConfFile, "out.dat")



import math

from past.utils import old_div


class SPGHistogram:
    __boxsize = 1
    __dict = {}
    __elements = 0

    def __init__(self, boxsize=1):
        self.__boxsize = boxsize
        self.__elements = 0.0

    #    import sys
    #    sys.stderr.write("histogram box size = %lf\n"%self.__boxsize)

    def __getitem__(self, value):

        nearest = self.__get_nearest(value)

        if nearest in self.__dict:
            return old_div(self.__dict[nearest], self.__elements)
        else:
            return 0.0

    def add_value(self, value):
        nearest = self.__get_nearest(value)
        if nearest in self.__dict:
            self.__dict[nearest] += 1.0
        else:
            self.__dict[nearest] = 1.0
        self.__elements += 1.0

    def __str__(self):
        if self.__elements == 0:
            return " "
        keys = list(self.__dict.keys())
        keys.sort()
        return "\n".join(
            [f"{k:f}\t{old_div(self.__dict[k], self.__elements):f}" for k in keys]
        )

    def __get_nearest(self, value):
        return self.__boxsize * math.floor(old_div(value, self.__boxsize))

    def get_dataset(self):
        if self.__elements == 0:
            return [[0, 0]]
        keys = list(self.__dict.keys())
        keys.sort()
        return [[k, (old_div(self.__dict[k], self.__elements))] for k in keys]


if __name__ == "__main__":
    a = SPGHistogram(0.1)
    import random
    import sys
    import time

    time1 = time.time()
    for i in range(100000):
        a.add_value(random.gauss(0.1, 0.5))
    time2 = time.time()

    print(a)
    sys.stderr.write(f"ellapsed time: {time2 - time1:f}\n")



"""
:::~ Author: Claudio Juan Tessone <tessonec@imedea.uib.es> (c) 2002-2005

Distributed According to GNU Generic Purpose License (GPL)
Please visit http://www.gnu.org
"""

# :::~ Important: for constants and functions already defined
import copy
import sys
from math import *

from . import ParamIterators


class ParamParser:
    """
    Initialized with a list of strings, each one containing commands.
    Each line will have a syntax as follows
    (iterator_type) (variable_name) [bounds]
    (iterator_type) can be one of the following characters
    '+' '-' '*' '/'  => all of them expect bounds given by [min_value]
                        [max_value] [step]
    '.'              => punctual iterator, (bounds) is in fact a (BLANK
                        separated) list with all the possible values
    ':'              => defines a CONSTANT (i.e. not iterable object)
    '#'              => repetition operator
    """

    # :::~ This variable specifies the list separator used while parsing
    # "param.dat"
    separator = " "

    # :::~ A dictionary with all the possible iterators
    iterator_types_dict = {
        "+": ParamIterators.ItOperatorPlus,
        "-": ParamIterators.ItOperatorMinus,
        "*": ParamIterators.ItOperatorProduct,
        "/": ParamIterators.ItOperatorDivision,
        "**": ParamIterators.ItOperatorPower,
        ":": ParamIterators.ItConstant,
        ".": ParamIterators.ItPunctual,
        "#": ParamIterators.ItRepetition,
    }

    # :::~ a list with an alternative order if your binary does not read
    # a free-style input file alternative_order = []
    # :::~ a list with all the varying entities (i.e. those not CONSTANT)
    # isvariable = []
    def __init__(self, lsLines):
        """
        lsLines is a list of commands understood by this class.
        """
        self.entities = []
        self.iterator_list = []
        self.variables_list = []
        self.actual_values = {}
        self.__parse(lsLines)

        self.reversed = copy.copy(self.iterator_list)
        self.reversed.reverse()
        self.reset()

    def __get_iteration_and_command(self, cadena):
        """
        returns the iteration type of a command. The iteration type is defined
        as the set of non alphanumeric characters at the beginning of the line
        """
        last_char = 0
        while not cadena[last_char].isalnum():
            last_char += 1
        return cadena[:last_char], cadena[last_char:]

    def __parse(self, ls):
        """
        internal function that parses the input
        """
        for sit in ls:
            # strips trailing and leading blanks
            # iteration type
            it_type, str_rest = self.__get_iteration_and_command(sit.strip())
            new_iterator = self.iterator_types_dict[it_type]()
            new_iterator.set_command(str_rest, self.separator)
            self.iterator_list.append(new_iterator)

        self.variables_list = [
            i_iter for i_iter in self.iterator_list if i_iter.is_variable()
        ]
        self.entities = [i_iter.get_varname() for i_iter in self.iterator_list]

    def __next__(self):
        """
        next() iterates over the possible values raising a StopIteration when
        the possible values are exhausted
        """
        if self.is_reset:
            self.is_reset = False
            return self.actual_values

        for i_iter in self.reversed:
            last_iterated = i_iter
            varname = i_iter.get_varname()
            try:
                self.actual_values[varname] = next(i_iter)

                break
            except StopIteration:
                self.actual_values[varname] = i_iter.reset()

                if last_iterated == self.iterator_list[0]:
                    raise StopIteration

        return self.actual_values

    # 1
    def reset(self):
        """
        This function resets the iterator to its starting point
        """
        for i_iter in self.iterator_list:
            i_iter.reset()
            self.actual_values[i_iter.get_varname()] = i_iter.get_value()
        self.is_reset = True

    def __iter__(self):
        return self

    def __str__(self):
        """
        defines how the actual value of the parameter set is printed out.
        A good candidate to be overwritten in inheriting classes.
        """
        thisstr = (
            "\n".join(
                [
                    f"{k}{self.separator}{self.actual_values[k]}"
                    for k in self.entities
                    if k
                ]
            )
            + "\n"
        )
        # :::~ replaces structures of the kind {var} by var-value, very useful
        # for generation of multiple output files.
        for i_iter in self.iterator_list:
            varname = i_iter.get_varname()
            thisstr = thisstr.replace(
                f"{varname}", f"{varname}-{self.actual_values[varname]}"
            )

        return thisstr

    def value_of(self, varn):
        """
        returns the actual value of the variable 'varn'
        """
        try:
            return self.actual_values[varn]
        except ValueError:

            sys.stderr.write(
                f"""'{varn}' not found among entities
"""
            )
            sys.exit()

    def set_order(self, new_order):
        """
        sets a new order for the output.
        May be a subset of the variables, but it can not be a superset
        useful if your program only reads a fixed input file
        """
        try:
            for i in new_order:
                [k.get_varname() for k in self.iterator_list].index(i)
        except ValueError:
            sys.stderr.write(
                f"""error! {i} not found among entities
entities = {self.entities}"""
            )
            sys.exit()
        self.entities = new_order

    # 1
    def directory_tree(self, limit=-1):
        """
        returns the directory path conducting to the actual values of the
        parameter set.  by default (limit=-1) the directory tree is extended to
        all the variables list except for the last variable.

        By setting limit to something else, you change the amount of variables
        kept left from the directory generation. (i.e. limit=-2, will leave out
        of the directory path the last two variables)

        """

        import os.path

        thepath = os.path.curdir + os.path.sep

        for i_iter in self.variables_list[:limit]:
            thepath += "{}-{}{}".format(
                i_iter.get_varname(),
                self.actual_values[i_iter.get_varname()],
                os.path.sep,
            )

        return thepath

    # 1
    def output_tree(self, limit=-1):
        """
        returns the output from limit given the actual values of the parameter set.
        by default (limit=-1) the output will only print the last variable value.
        By setting limit to something else, you change the amount of variables printed
        (i.e. limit=-2, will print the value of the last two variables)
        """
        theoutput = ""
        for i_iter in self.variables_list[limit:]:
            theoutput += f"{self.actual_values[i_iter.get_varname()]}" + self.separator
        return theoutput


#
#
if __name__ == "__main__":
    # command=[":a 0","*c 2 6 3",":filename {c}.dat","+d 5 9 4","#2",":end"]
    #  command=["*c 2 18 3"]
    command = [":a 0", "*c 2 6 3", ":filename {c}.dat", "+d 5 9 4", ":end"]
    #
    print(command)
    pp = ParamParser(command)
    # pp.set_order(["c","a","d"])
    for i in pp:
        print(pp)



import sys
from math import *

from past.utils import old_div

from .Load import *
from .ParamParser import *

#
#
# :::~ Author: Claudio Juan Tessone <tessonec@imedea.uib.es> (c) 2002-2003
#
# Distributed According to GNU Generic Purpose License (GPL)
# Please visit www.gnu.org
#
#


class MeanCalculation:

    """
    When a simulation is run with a given number of repetitions what is needed is a
    tool for calculating the mean values. This is that tool
    """

    def __init__(self, lsLines):
        ls2 = [i for i in lsLines if i.strip()[0] != "#"]

        self.pp_varying = ParamParser(ls2)

    def searchinfile(self, fname, st):
        return [i.split() for i in open(fname).readlines() if i.split()[0] == st]

    def mean(self, fin_name="out.dat", fout_name="out.mean"):

        dirname = self.pp_varying.directory_tree()

        fsearch = f"{dirname}{fin_name}"
        fout = f"{dirname}{fout_name}"

        try:
            all_data = loadData(fsearch)
        except:
            print(f"Error! '{fsearch}' file doesn't exist or not enough permissions...")
            sys.exit()

        try:
            columnas = list(range(len(all_data[0])))
        except:

            sys.stderr.write(f"mean-> error! in directory '{dirname}'\n")
            sys.stderr.write(f"all_data = {str(all_data)}\n")

        tmpDict = {}

        for i in all_data:
            tmpDict[i[0]] = 0
        valuesX = list(tmpDict.keys())
        valuesX.sort()

        xDict = {}
        nPoints = {}

        import copy

        zeros = [0 for i in columnas]

        for i in valuesX:
            xDict[i] = copy.copy(zeros)
            nPoints[i] = 0.0

        for row in all_data:
            if len(row) == len(all_data[0]):
                nPoints[row[0]] += 1.0
                vec = xDict[row[0]]
                for j in columnas:
                    vec[j] += row[j]

        dataOut = []

        for x in valuesX:
            if nPoints[x] != 0:
                dataOut.append([old_div(xDict[x][j], nPoints[x]) for j in columnas])

        try:
            dumpData(fout, dataOut)
        except:
            print(f"Error! Couldn't create output file '{fsearch}' ...")
            sys.exit()

    def doit(self, fin_name="out.dat", fout_name="mean.dat"):
        for i in self.pp_varying:
            self.mean(fin_name, fout_name)


#
#



#
# Specifies the name of the last variable considered
# before plotting
#
lastvar = "m"

#
# Specifies the columns to be plotted
# and its label (in grace format)
#
col_contents = [(1, 2, "\\f{Symbol}\\l{.15}r"), (3, 4, "\\f{Times-Italic}T")]


#
# Specifies the grace format of the x axis
#
xlabel = "\\f{Times-Italic}P"
